@@ -1,6 +1,12 @@
 
 import os
 
+from subnet import (
+    IPv4Address,
+    IPv6Address,
+    ip_address,
+)
+
 from .base import (
     WireGuardBase,
     MAX_ADDRESS_RETRIES,
@@ -57,7 +63,7 @@ class WireGuardServer(WireGuardBase):
         if item == self.private_key:
             return True
 
-        return item in [peer.private_key for peer in self.peers]
+        return item in self.peers_privkeys
 
     def address_exists(self, item):
         """
@@ -70,7 +76,26 @@ class WireGuardServer(WireGuardBase):
         if item == self.address:
             return True
 
-        return item in [peer.address for peer in self.peers]
+        return item in self.peers_addresses
+
+    @property
+    def peers_addresses(self):
+        if not self.peers:
+            return []
+        return [peer.address for peer in self.peers]
+
+    @property
+    def peers_privkeys(self):
+        if not self.peers:
+            return []
+        return [peer.private_key for peer in self.peers]
+
+    @property
+    def _default_peers_config_file(self):
+        return os.path.join(
+                self.config_path,
+                f'{self.interface}-peers.conf',
+            )
 
     @property
     def peers_config_file(self):
@@ -78,7 +103,11 @@ class WireGuardServer(WireGuardBase):
         Returns the peers config file name, if appropriate
         """
 
-        return self._peers_config_file
+        if isinstance(self._peers_config_file, str):
+            return self._peers_config_file
+        if self._peers_config_file == True:
+            return self._default_peers_config_filename
+        return None
 
     @peers_config_file.setter
     def peers_config_file(self, value):
@@ -86,13 +115,10 @@ class WireGuardServer(WireGuardBase):
         Sets the peers config file
         """
 
-        if isinstance(value, bool):
-            self._peers_config_file = os.path.join(
-                self.config_path,
-                f'{self.interface}-peers.conf',
-            )
+        if value == True:
+            self._peers_config_file = self._default_peers_config_filename
 
-        elif not isinstance(value, str):
+        elif not isinstance(value, str) and value is not None:
             raise ValueError('Invalid value for peers_config_file: %s' % value)
 
         else:
@@ -134,37 +160,56 @@ class WireGuardServer(WireGuardBase):
         and optionally updating the peer's data to obtain uniqueness
         """
 
+        if (self.unique_address(peer, max_address_retries=max_address_retries) and
+            self.unique_privkey(peer, max_privkey_retries=max_privkey_retries)):
+            self.peers.append(peer)
+
+        else:
+            raise ValueError('Could not add peer to this server. It is not unique.')
+
+    def unique_address(self, peer, max_address_retries=None):
+        if peer in self.peers:
+            return True
+
         if max_address_retries is None or max_address_retries == True:
             max_address_retries = MAX_ADDRESS_RETRIES
-        elif max_address_retries == False:
-            max_address_retries = 0
+
+        if not max_address_retries:
+            if self.address_exists(peer.address):
+                raise ValueError(f'IP address is already used on this server: {peer.name} ({peer.address})')
+
+        else:
+            count = 0
+            while self.address_exists(peer.address):
+                if count >= max_address_retries:
+                    raise ValueError(f'Too many retries to obtain an unused IP address: {peer.name}')
+
+                peer.address = self.subnet.random_ip()
+                count += 1
+
+        return True
+
+    def unique_privkey(self, peer, max_privkey_retries=None):
+        if peer in self.peers:
+            return True
 
         if max_privkey_retries is None or max_privkey_retries == True:
             max_privkey_retries = MAX_PRIVKEY_RETRIES
-        elif max_privkey_retries == False:
-            max_privkey_retries = 0
 
-        count = 0
-        while self.address_exists(peer.address):
-            if max_address_retries == 0:
-                raise ValueError(f'IP address is already used on this server: {peer.name} ({peer.address})')
-            elif count >= max_address_retries:
-                raise ValueError(f'Too many retries to obtain an unused IP address: {peer.name}')
-
-            peer.address = self.subnet.random_ip()
-            count += 1
-
-        count = 0
-        while self.privkey_exists(peer.private_key):
-            if max_privkey_retries:
+        if not max_privkey_retries:
+            if self.privkey_exists(peer.private_key):
                 raise ValueError(f'Private key is already used on this server: {peer.name}')
-            elif count >= max_privkey_retries:
-                raise ValueError(f'Too many retries to obtain an unused private key: {peer.name}')
 
-            peer.private_key = generate_key()
-            count += 1
+        else:
+            count = 0
+            while self.privkey_exists(peer.private_key):
+                if count >= max_privkey_retries:
+                    raise ValueError(f'Too many retries to obtain an unused private key: {peer.name}')
 
-        self.peers.append(peer)
+                peer.private_key = generate_key()
+                count += 1
+
+        return True
 
     def config(self):
         """
@@ -207,8 +252,8 @@ PostUp = wg addconf %i {self.peers_config_file}
         """
 
         config = ''
-        for peer in self.peers.values():
-            config += peer.server_config()
+        for peer in self.peers:
+            config += peer.serverside_config()
 
         return config
 
